//////////////////////////////////////////////////////////
////	Lecture de fichier de position de trajectoire ////
////	et calcul  de trajectoire en temps rel 	  ////
////	par la mthode Catmull						  ////
//////////////////////////////////////////////////////////

#include "catmull.h"

////////////////////////////////////////////////////////////
//	Chargement d'un fichier de trajectoire          	  //
//		File_name  - Nom du fichier					      //
//		FinalTrajectory	- Une trajectoire  initialiser	  //
//  Retourne :											  //
//		Void											  //
////////////////////////////////////////////////////////////
void LoadObjectPath(char *File_name, MyTrajectory *FinalTrajectory)
{
	FILE *file;				// Le fichier
	MyPathList *tempPath;	// pointeur temporaire vers la liste de positions
	int i=0;				// gre la boucle de lecture des positions

	// on initialise par defaut les variables de la structure au cas o...
	FinalTrajectory->loop = 0;			// Animation boucle?
	FinalTrajectory->NbPositions = 0;	// Nbr de position
	FinalTrajectory->speed = 0.0;		// Vitesse
	FinalTrajectory->startTime = 0.0;	// Temps  couler avant le dpart 
	FinalTrajectory->time = 0.0;		// temps pour l'horloge interne du calcul catmull

	file=fopen(File_name,"r");	// Ouverture du fichier

	while(fgetc(file) != '\n');

	fscanf(file, "POSITIONS %d\n", &FinalTrajectory->NbPositions); // Nombre de positions
	fscanf(file, "LOOP %d\n", &FinalTrajectory->loop); // Animation boucle? 0=non, 1=oui
	fscanf(file, "SPEED %f\n", &FinalTrajectory->speed); // Vitesse de l'animation
	fscanf(file, "START %f\n", &FinalTrajectory->startTime); // temps  couler avant dpart

	// allocation dynamique de la 1re structure de la liste des positions 
	FinalTrajectory->firstPosition=(MyPathList*)malloc(sizeof(MyPathList));
	FinalTrajectory->firstPosition->next=NULL;
	tempPath=FinalTrajectory->firstPosition;

	// stockage de toutes les positions une  une par allocation dynamique
	while(i < FinalTrajectory->NbPositions) 
	{
		if(i > 0)	// car la premire structure de la liste est dja alloue
		{
			tempPath->next = (MyPathList*)malloc(sizeof(MyPathList));
			tempPath = tempPath->next;
			tempPath->next = NULL;
		}

		fscanf(file, "\n");
		fscanf(file, "%f %f %f\n",&tempPath->x,&tempPath->y,&tempPath->z);

		++i;
	}

	fclose(file);		// fermeture du fichier

	if(FinalTrajectory->loop==1)	// si animation boucle, on boucle la liste
		tempPath->next=FinalTrajectory->firstPosition;	// dernier lment=premier lment

	// path pointera sur la liste, firstPosition pointera toujours vers le dbut de la liste
	FinalTrajectory->path=FinalTrajectory->firstPosition;	
}



//////////////////////////////////////////////////////////////////
//	Avance d'un point de contrle sur la trajectoire si			//
//  une seconde s'est coule									//
//		value	- temps coul depuis la dernire boucle		//
//				  de temps										//
//		objPosition - la trajectoire avec sa liste de positions //
//	Retourne:													//
//		Void													//
//////////////////////////////////////////////////////////////////
void CatmullUpdate(float value, MyTrajectory *objPosition)
{
	objPosition->time += value;
		
	if (objPosition->time > 1.0f)	// il est temps d'avancer dans la liste
	{	
		// Catmull ncessite 4 points pour faire un calcul
		if (objPosition->path->next->next->next->next)
		{
			objPosition->time = objPosition->time - 1.0f;
			objPosition->path = objPosition->path->next;
		}
		else	// on arrive  la fin de la trajectoire on bloque l'horloge de Catmull
			objPosition->time = 1.0f;	
	}
}


///////////////////////////////////////////////////////////////////
//	Calcul et renvoit la position d'un objet  partir d'une 	 //
//  trajectoire lisse par Catmull								 //
//		objPosition - La trajectoire avec sa liste de points	 //
//	Retourne													 //
//		Void													 //
///////////////////////////////////////////////////////////////////
void CatmullGivePosition(MyTrajectory *objPosition)
{
	int i;					// compteur par defaut
	g3dVector posIn[4], targIn[4], rollIn[4]; 	// pour les 4 points de contrle permettant le calcul
	MyPathList *givePath;	// liste des positions (temporaire)

	float t,t2, t3, t1;
	float a, b, c, d;

	t  = objPosition->time;
	t2 = t*t;			
	t3 = t*t*t;
	t1 = (1-t)*(1-t);

	a = -t*t1;
	b = 2-5*t2+3*t3;
	c = t*(1+4*t-3*t2);
	d = t2*(1-t);

	// On rcupre 4 points de contrls successifs de la liste de la trajectoire
	if (objPosition->path->next->next->next) // existent-ils vraiment?
	{
		givePath = objPosition->path;

		for (i = 0; i < 4; i++)
		{
			posIn[i].x = givePath->x;
			posIn[i].y = givePath->y;
			posIn[i].z = givePath->z;
			givePath = givePath->next;
		}
	}
	else // Cette condition ne s'applique jamais sauf erreur grave de mmoire
	{	
		printf("Erreur de gestion de la mmoire!!!\n");
		objPosition->path = objPosition->firstPosition;
		return;
	}

	objPosition->x = (a*posIn[0].x + b*posIn[1].x + c*posIn[2].x - d*posIn[3].x)/2;
	objPosition->y = (a*posIn[0].y + b*posIn[1].y + c*posIn[2].y - d*posIn[3].y)/2;
	objPosition->z = (a*posIn[0].z + b*posIn[1].z + c*posIn[2].z - d*posIn[3].z)/2;
}

//////////////////////////////////////////////////////////////////
//	Libre la mmoire alloue aux listes de la structure de		//
//  trajectoire													//
//		FinalTrajectory	- Une trajectoire  librer			    //
//	Retourne:													//
//		Void													//
//////////////////////////////////////////////////////////////////
void CatmullFree(MyTrajectory *ListToFree)
{
	int i = 1;			// contrle le nbr de free  effectuer
	int j = 0;			// permet d'avancer dans la liste
	MyPathList *tempPath;	// pointeur temporaire vers la liste de positions

	while( i <= ListToFree->NbPositions )
	{
		tempPath=ListToFree->firstPosition;		// on part du dbut de la liste
		while( j < ListToFree->NbPositions - i )// jusqu' la fin de la liste
		{		
			tempPath=tempPath->next;		// on avance
			++j;				// incrmentation du compteur d'avancement
		}

		free(tempPath);			// libration mmoire de l'lment de la liste
		++i;					// incrementation du compteur de contrle
		j=0;					// mise  zero du compteur d'avancement
	}
}


